# Axe Interpreter #


## Table of Contents ##

*   Synopsis
*   Thanks and credits
*   Running this program
*   Restrictions and limitations
*   Allowed tokens
*   Version history and roadmap


## Synopsis ##

Axe Interpreter is a version of Axe meant to be run on computers.  This is
currently under heavy development, and will allow only a _very_ limited subset
of Axe to be used.  I created this program because I was...

1.  Bored.
2.  Wanted to try making a decent cross-platform development platform for Axe.

This program is written in Python, and should be compatible with all major
operating systems, provided they have Python installed.

Author:   Michael Lee <michael.lee.0x2a@gmail.com>
License:  GNU Lesser GPL
Version:  0.0
Released: December 20th, 2011


## Thanks and credits ##

*Kevin Horowitz (a.k.a. Quigibo):*  For making Axe Parser.
*The Omnimaga Community:*           For support and such
*PLY (Python Lex Yacc):*            For providing lexing and parsing tools
*Pygame:*                           For graphics


## Running this program ##

You will need the following:

*   Python 2.7 <http://python.org>
*   Pygame 1.9.1+ <http://pygame.org>

In addition, this program also uses PLY 3.4 <http://www.dabeaz.com/ply/>, but 
you don't need to install that: it should be included.

This program is run via Command Prompt.  To run the interpreter, type:

    $ python main.py --interpret

To execute a program, type:

    $ python main.py myprogram.txt
    
If you want to test specific components of the program, type one of the below:

    $ python main.py --test lex
    $ python main.py --test parse
    $ python main.py --test io



## Restrictions and limitations ##

I only started working on this a few days ago.  There are loads of limitations
and restrictions.

*   You can only use the variables A to Z
*   You can only use numbers.  Strings, hex, binary, etc. are not supported
*   There are no pointers yet
*   You can't declare constants yet
*   Only a limited subset of tokens are currently implemented (see below)
*   No grayscale is supported.


## Allowed tokens ##

Currently, only a very limited subset of tokens are allowed.  Currently allowed
tokens are...

### Control Structures ###
*   If          (The tokens 'Else', '!If', etc. are not implemented yet)
*   End
*   For         ('For(A,0,10)' is supported, 'For(10)' is not.)
*   While
*   Repeat

### Input/Output ###
*   Disp        (currently prints to the command line)
*   Rect
*   RectI
*   DispGraph
*   ClrDraw
*   Line
*   Pxl-On
*   Pxl-Off

### Math ###
*   +           (add)
*   -           (subtract)
*   *           (multiply)
*   /           (divide)
*   ->          (assignment (STO key))

### Comparison ###
*   >           (greater then)
*   <           (less then)
*   >=          (greater then or equal to)
*   <=          (less then or equal to)
*   =           (equal to)
*   !=          (not equal to)

### Meta ###

Meta tokens are ones which are not implemented in Axe, but do stuff to this
interpreter in particular.  Meta tokens work in both in interpreter mode and 
while running a file.

*   @EXIT
    Ends the program in both interpreter and drawing mode.
    
*   @STATE
    Displays to the command line all variables.
    
*   @DRAW
    If turned on, displays all drawing commands the moment they're called 
    without having to call 'DispGraph'.  Off by default while running a text
    file, on by default in interpreter mode.  '@DRAW 1' to turn on, and 
    '@DRAW 0' to turn off.
    
*   @HELP
    Displays a brief help message.
    
*   @ABOUT
    Displays a brief message about this program.
    
*   @DEBUG
    Displays extra info to the command line for debugging.  Type '@DEBUG 1' 
    to turn it on and '@DEBUG 0' to turn it off.  Off by default, you shouldn't
    need this token.

### Misc. ###

You can use both colons and newlines to signify the end of a line.  Therefore,

    If A:Pxl-On(10,20):End

...is equivalent to...

    If A
        Pxl-On(10,20)
    End



## Version history and roadmap ##

### Version History ###

*   0.0     December 20th, 2011
    This release.  Has only numbers, rudimentary drawing primitives, and 
    a few control structures


### Roadmap ###

*   GetKey and more input.
*   Adding actual memory handling, pointers, and constants.
*   Attempting to integrate the memory and Pygame buffers to each other
*   Adding more drawing commands
*   Adding more control structures
*   Optimizing
*   Grayscale


