#!/usr/bin/env python
"""
Axe Interpreter

An interpreter for programs written in the Axe Parser language for computers.

Author: Michael Lee (<michael.lee.0x2a@gmail.com>)
License: GNU Lesser GPL
"""

# System libraries
import sys

# 3rd party libraries

# My libraries
import io
import axelex
import axeparse
import axeinterpreter
from meta import *

argparser = io.CommandIO()
options = argparser.parse()


class Axe():
    def __init__(self):
        self.lexer = axelex.build()
        self.parser = axeparse.build()
        self.interpreter = axeinterpreter.build()
        return
    
    def run(self, text):
        assert(isinstance(text, basestring))
        if text[-1] != '\n' or text[-1] != ':':
            text = text + '\n'
        result = self.parser.parse(text, lexer=self.lexer)
        return self.interpreter.run(result)


if options['interpret']:
    axeinterpreter.test_interpreter()
elif options['test']:
    choice = options['test']
    if options['input_path']:
        with open(options['input_path'], 'r') as file_name:
            text = file_name.read()
    else:
        text = ''
    if choice == 'io':
        io.test_emulator()
    elif choice == 'lex':
        axelex.test_lexer(text=text)
    elif choice == 'parse':
        axeparse.test_parser(text=text)
    elif choice == 'interpret':
        axeinterpreter.test_interpreter()
else:
    with open(options['input_path'], 'r') as file_name:
        text = file_name.read()
    axe = Axe()
    try:
        output = axe.run(text)
        print output
    except (ExitException, KeyboardInterrupt):
        print '\nEnding...\n'



